package org.apkplug.Bundle;

import java.util.Map;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
/**
 * 插件安装服务
 * @author 梁前武
 * www.apkplug.com
 * v2.0.0
 */
public interface BundleControl {
	/**
	 * 安装/更新插件
	 * @param context       
	 * @param loaclFilePath   本地插件文件的决定路径 (apkplug2.6.4前apk文件地址前须加"file:")
	 * @param callback        安装过程回调接口
	 * @return                安装时匹配版本  如果版本相同停止安装/更新
	 * @throws Exception
	 */
	Bundle install(BundleContext context,String loaclFilePath,installCallback callback) throws Exception;
	/**
	 * 安装/更新插件
	 * @param context
	 * @param loaclFilePath   本地插件文件的决定路径 (apkplug2.6.4前apk文件地址前须加"file:")
	 * @param callback        安装过程回调接口
	 * @param startlevel      插件启动级别      小于default_startlevel的插件将在框架启动时启动  大于等于default_startlevel的将不自动启动  default_startlevel=2
	 * @param isCheckVersion  安装时是否匹配版本  true 如果版本相同停止安装/更新  false 版本相同卸载后重新安装  
	 * @return
	 * @throws Exception
	 */
	Bundle install(BundleContext context,String loaclFilePath,installCallback callback,int startlevel,boolean isCheckVersion) throws Exception;
	/**
	 * 安装/更新插件
	 * @param context
	 * @param loaclFilePath            本地插件文件的决定路径 (apkplug2.6.4前apk文件地址前须加"file:")
	 * @param callback                 安装过程回调接口
	 * @param startlevel               插件启动级别     小于default_startlevel的插件将在框架启动时启动  大于等于default_startlevel的将不自动启动  default_startlevel=2
	 * @param isCheckVersion           安装时是否匹配版本  true 如果版本相同停止安装/更新  false 版本相同卸载后重新安装 
	 * @param Check_Main_Signature     插件与宿主签名是否必须一致   默认为false【可以不一致】
	 * @param Check_Bundle_Signature   插件更新时与旧版插件签名是否必须一致   默认为false【可以不一致】
	 * @return
	 * @throws Exception
	 */
	Bundle install(BundleContext context,String loaclFilePath,installCallback callback,int startlevel,boolean isCheckVersion,boolean Check_Main_Signature,boolean Check_Bundle_Signature) throws Exception;
	
	
	/**
	 * 安装/更新插件
	 * @param context
	 * @param installinfo              安装/更新插件的详细参数  【使用这个类的原因是参数过多】
	 * @param callback                 安装过程回调接口
	 * @return
	 * @throws Exception
	 */
	Bundle install(BundleContext context,InstallInfo installinfo,installCallback callback) throws Exception;
	
	/**
	 * 获取插件文件plugin.xml中的属性
	 * @param context
	 * @param loaclFilePath
	 * @return 如果不存在assets/plugin.xml 或文件不正确返回null
	 * @throws Exception
	 */
	Map BundleManifest(BundleContext context,String loaclFilePath) throws Exception;
}	
