package org.apkplug.Bundle;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apkplug.Bundle.BundleControl;
import org.apkplug.Bundle.OSGIServiceAgent;
import org.apkplug.Bundle.installCallback;
import org.osgi.framework.BundleContext;

import android.content.SharedPreferences;
/**
 * 插件安装封装器
 * 支持
 * 1.安装 assets目录下的apk   installForAssets
 * 2.以及文件系统中的apk文件    installBundleFile
 * @author 梁前武
 */
public class InstallBundler {
	private SharedPreferences sp;
	private SharedPreferences.Editor editor;
	public boolean DEBUG=false;
	private OSGIServiceAgent<BundleControl> agent=null;
	private BundleContext mcontext=null;
	/**
	 * @return mcontext
	 */
	public BundleContext getMcontext() {
		return mcontext;
	}
	public InstallBundler(BundleContext mcontext){
		this.mcontext=mcontext;
		agent=new OSGIServiceAgent<BundleControl>(mcontext,BundleControl.class);
		sp = mcontext.getAndroidContext().getSharedPreferences("apkpluginstallconfig.ini",0);
		editor=sp.edit();
	}
	public void putString(String key, String value){
		editor.putString(key, value);
		editor.commit();
	}
	public String getString(String key, String value){
		return sp.getString(key,value);
	}
	/**
	 *  例子:
	 *  把BundleDemo1.apk从assets文件夹中移至应用安装目录中 
	 *	this.installForAssets("VehicleBundle.apk", "1.9.0", callback,null);
	 *
	 * @param mcontext    
	 * @param plugfile        assets目录下的文件名  如 drag-sort-listview.apk
	 * @param version         当前安装的插件版本号   如下次更新宿主时，可以提高这个版本好，以让框架安装最新宿主assets目录下的插件
	 * @param callback        安装事件回掉接口
	 * @param installinfo     安装参数  如果为null就使用默认的安装参数
	 * @throws Exception
	 */
	public void installForAssets(String plugfile,String version,InstallInfo installinfo,installCallback callback) {
		// startlevel设置为2插件不会自启 isCheckVersion不检测插件版本覆盖更新
		File f1=null;
		if(installinfo==null){
			installinfo=new InstallInfo();
		}
		try {
				
				if(!DEBUG){
					//不是调试模式
					String PlugVersion=this.getString(plugfile,null);
					if(PlugVersion!=null){
						if(PlugVersion.equals(version)){
							//如果本地已安装的插件版本等与目前插件的版本，那么就不安装了
							return ;
						}
					}
					InputStream in=mcontext.getAndroidContext().getAssets().open(plugfile);
					f1=new File(mcontext.getAndroidContext().getFilesDir(),plugfile);
					if(!f1.exists()){
						copy(in, f1);
					}
					installinfo.setApkFilePath("file:"+f1.getAbsolutePath());
					agent.getService().install(mcontext,installinfo,callback);
					//安装完成后删除文件
					f1.delete();
					this.putString(plugfile, version);
				}else{
					InputStream in=mcontext.getAndroidContext().getAssets().open(plugfile);
					f1=new File(mcontext.getAndroidContext().getFilesDir(),plugfile);
					copy(in, f1);
					installinfo.setApkFilePath("file:"+f1.getAbsolutePath());
					agent.getService().install(mcontext, installinfo,callback);
				}
				
		} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
		} catch (Exception e) {
				// TODO 自动生成的 catch 块
				e.printStackTrace();
		} 
	}

	/**
	 * 安装apk文件
	 * @param installinfo
	 * @param callback
	 * @throws Exception
	 */
	public void installBundleFile(InstallInfo installinfo,installCallback callback) throws Exception{
		agent.getService().install(mcontext, installinfo,callback);
	}
	private void copy(InputStream is, File outputFile)
	        throws IOException
	    {
	        OutputStream os = null;

	        try
	        {
	            os = new BufferedOutputStream(
	                new FileOutputStream(outputFile),4096);
	            byte[] b = new byte[4096];
	            int len = 0;
	            while ((len = is.read(b)) != -1)
	                os.write(b, 0, len);
	        }
	        finally
	        {
	            if (is != null) is.close();
	            if (os != null) os.close();
	        }
	    }
}
