package org.apkplug.Bundle;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
/**
 * OSGI服务查询代理,简化查询流程
 * @author 梁前武
 * www.apkplug.com
 * @param <T>  OSGI服务Class
 * v2.0.0
 */
public class OSGIServiceAgent<T> {
	public static int cache=0;
	public static int real_time=1;
	private T service=null;
	private Class servicename=null;
	private int type=cache;
	private BundleContext context=null;
	private String filter=null;
	/**
	 * 
	 * @param context      插件上下文
	 * @param servicename  OSGI服务Class
	 */
	public OSGIServiceAgent(BundleContext context,Class servicename){
		this(context,servicename,cache);
	}
	/**
	 * 用OSGI服务的Class 作为查询条件
	 * @param context     插件上下文
	 * @param servicename OSGI服务Class
	 * @param type        查询类型 
	 *  Cache               查询到服务类后便缓存上下次获取不再查询  (适合系统服务)
	 *  real_time           实时查询    每次获取都查询一次         (适合普通,随时可能注销的服务)
	 */
	public OSGIServiceAgent(BundleContext context,Class servicename,int type){
		this.servicename=servicename;
		this.type=type;
		this.context=context;
	}
	/**
	 * 用OSGI服务的Class + 过滤器 作为查询条件
	 * @param context       插件上下文
	 * @param servicename   OSGI 服务名
	 * @param filter        查询过滤器
	 * @param type          查询类型
	 *  Cache               查询到服务类后便缓存上下次获取不再查询  (适合系统服务)
	 *  real_time           实时查询    每次获取都查询一次         (适合普通,随时可能注销的服务)
	 */
	public OSGIServiceAgent(BundleContext context,Class servicename,String filter,int type){
		this.servicename=servicename;
		this.type=type;
		this.context=context;
		this.filter=filter;
	}
	/**
	 * 获取服务实例，如果查询成功返回实例，查询失败返回null 或报异常
	 * @return
	 */
	public T getService()throws Exception{
		if(type==this.cache){
			if(service==null){
				service=search();
			}
			return service;
		}else if(type==this.real_time){
			return search();
		}else{
			return search();
		}
	}
	private T search() throws InvalidSyntaxException{
		if(context==null)
			return null;
		if(filter==null){
			ServiceReference reference=context.getServiceReference(servicename.getName());
	    	if(null!=reference){
	    		T service=(T)context.getService(reference);
	    		context.ungetService(reference);
	    		return service;
	    	}
	    	return null;
		}else{
			ServiceReference[] references=context.getServiceReferences(servicename.getName(),filter);
	    	if(null!=references){
	    		if(references.length>0){
		    		T service=(T)context.getService(references[0]);
		    		context.ungetService(references[0]);
		    		return service;
	    		}
	    	}
	    	return null;
		}
	}
}
