package org.apkplug.app;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Properties;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import org.json.JSONException;
import org.json.JSONObject;
import org.mytoole.util.Reflict.ReflictUtil;
import org.tengxin.bu;
import org.ungoverned.ExceptionHandler.LooperThreadFactory;
import org.ungoverned.ExceptionHandler.MyThreadGroup;
import org.ungoverned.apkplug.StringForm;
import org.ungoverned.moduleloader.Smith;

import dalvik.system.DexClassLoader;
import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.MessageQueue;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.WindowManager;
/**
 * 启动框架 工厂类
 * @author 梁前武 
 * www.apkplug.com
 * v2.0.0
 */
@SuppressLint("NewApi")
public class FrameworkFactory {
	
	private FrameworkInstance frame=null;
	private Properties prop=null;
	private static FrameworkFactory _instance=null;
	private String framework_version="2.7.1";
	//单例静态工厂方法,同步防止多线程环境同时执行 
	synchronized public static FrameworkFactory getInstance(){
	   if(_instance==null){
		   _instance=new FrameworkFactory();
	   }
	   return _instance;
	}
	private FrameworkFactory(){
		
	}
	
	
	private Properties getProperties(Context appContext) throws Exception{
		if(prop!=null)
			return prop;
		File properties=new File(appContext.getFilesDir(),"apkplug.properties");
		if(!properties.exists()){
			//本地的配置文件不存在
			java.io.InputStream ioproperties=this.getClass().getResourceAsStream("/assets/apkplug.properties");
			if(ioproperties!=null){
				copy(ioproperties,properties);
			}else{
				throw new Exception("no found apkplug.properties");
			}
		}
		prop = new Properties();
		prop.load(new FileInputStream(properties.getAbsoluteFile()));
		String oldframework_version=prop.getProperty("framework_version");
		
		if(oldframework_version.compareTo(framework_version)<0){
			//如果apkplug.properties中的版本小与当前框架版本号则需要更新apkplug.properties文件
			java.io.InputStream ioproperties=this.getClass().getResourceAsStream("/assets/apkplug.properties");
			if(ioproperties!=null){
				copy(ioproperties,properties);
			}else{
				throw new Exception("no found apkplug.properties");
			}
			//重新加载配置文件
			prop = new Properties();
			prop.load(new FileInputStream(properties.getAbsoluteFile()));
			return prop;
		}
		return prop;
	}
	/**
	 * 启动框架
	 * @param activatorList   启动框架时可以传人一个activator列表,作为系统级别的OSGI服务
	 * @param appContext      android的Context 建议使用ApplicationContext
	 * @return
	 * @throws Exception
	 */
	public FrameworkInstance start( List activatorList,Context appContext) throws Exception{
		return this.start(activatorList, appContext, new DefaultProperty(appContext));
	}
	/**
	 * 启动框架
	 * @param activatorList  启动框架时可以传人一个activator列表,作为系统级别的OSGI服务
	 * @param appContext     android的Context 建议使用ApplicationContext
	 * @param property		 
	 * @param metrics
	 * @return
	 * @throws Exception
	 */
	public FrameworkInstance start( List activatorList,Context appContext,PropertyInstance property) throws Exception{
		System.out.println(FromJNI(0));
		init.init(appContext,property);
		//DisplayMetrics dm2 = appContext.getResources().getDisplayMetrics();
		 DisplayMetrics dm2 = new DisplayMetrics();
		 
		WindowManager wm = (WindowManager)appContext.getSystemService(appContext.WINDOW_SERVICE);  
		android.view.Display display = wm.getDefaultDisplay();  
		display.getMetrics(dm2);
		return this.start(activatorList, appContext, property, dm2);
	}
	
	/**
	 * 启动框架
	 * @param activatorList  启动框架时可以传人一个activator列表,作为系统级别的OSGI服务
	 * @param appContext     android的Context 建议使用ApplicationContext
	 * @param property		 框架需要保持持久化信息所需要的接口
	 * @param metrics
	 * @return
	 * @throws Exception
	 */
	public FrameworkInstance start( List activatorList,Context appContext,PropertyInstance property,DisplayMetrics dm2) throws Exception{
	
		if(dm2==null){
			dm2 = new DisplayMetrics();//appContext.getResources().getDisplayMetrics();
			WindowManager wm = (WindowManager)appContext.getSystemService(appContext.WINDOW_SERVICE);  
			android.view.Display display = wm.getDefaultDisplay();  
			display.getMetrics(dm2);
		}
		//System.out.println(dm2);
		//return new OscarFramework(activatorList,appContext,property,metrics);
		if(frame==null){
			Class oscarClass=null;
			Properties prop =getProperties(appContext);
			String framework_version=prop.getProperty("framework_version");
			String minSOVersion=prop.getProperty("minSOVersion");
			String bundle_path=prop.getProperty("bundle_path","assets/Bundle2.7.1.hhm01.jar");
			String bundle_name=prop.getProperty("bundle_name","bundle2.7.1.01.jar");
			String classname=prop.getProperty("classname","org.tengxin.bu");
			File fdex=new File(appContext.getFilesDir(),bundle_name);
			if(!fdex.exists()){
				java.io.InputStream dex=this.getClass().getResourceAsStream("/"+bundle_path);
				
				if(dex!=null){
					copy(dex,fdex);
					DexClassLoader f=new DexClassLoader(fdex.getAbsolutePath(),appContext.getFilesDir().getAbsolutePath(),null,this.getClass().getClassLoader());
					oscarClass=f.loadClass(classname);
				}else{
					throw new Exception("no found dex");
				}
			}else{
				DexClassLoader f=new DexClassLoader(fdex.getAbsolutePath(),appContext.getFilesDir().getAbsolutePath(),null,this.getClass().getClassLoader());
				oscarClass=f.loadClass(classname);
			}
			
			if(oscarClass!=null)
				frame=(FrameworkInstance) oscarClass.newInstance();
			else
				throw new Exception("no found class");
			
			if(frame!=null)
				frame.start(activatorList, appContext, property, dm2);
		}
		//String classPath = System.getProperty("java.class.path", ".");
		//Log.e("classPath", classPath);
		//ReflictUtil.printInnerClass(ClassLoader.class);
		/*if(frame==null)
			frame=new bu();
		if(frame!=null)
			frame.start(activatorList, appContext, property, dm2);*/
		return frame;
	}
	/**
	 * 更新Bundle.jar包
	 * @param appContext
	 * @param bundlepath  最新Bundle.jar文件路径
	 * @return  如果成功将返回true 失败将报异常 (可捕获异常判断是否更新成功)
	 * @throws IOException
	 */
	public boolean updata(Context appContext,String bundlepath) throws IOException{
		Properties propold=null;
        try {
			propold =getProperties(appContext);
		} catch (Exception e) {
			// TODO 自动生成的 catch 块
			e.printStackTrace();
		}
		File bundleJar = new File(bundlepath);
		JarFile jarFile= new JarFile(bundleJar);
		ZipEntry ze=jarFile.getEntry("assets/apkplug.properties");
		if (ze == null)
        {
            throw new IOException("No apkplug.properties: assets/apkplug.properties" );
        }
		BufferedInputStream is = new BufferedInputStream(jarFile.getInputStream(ze));
        if (is == null)
        {
        	throw new IOException("No apkplug.properties: assets/apkplug.properties");
        }
        File properties=new File(appContext.getFilesDir(),"apkplug.properties");
        if(properties.exists()){
        	//properties.delete();
        }
        	
        // Create the file.
        copy(is, properties);
        
        prop = new Properties();
		prop.load(new FileInputStream(properties.getAbsoluteFile()));
		String framework_version=prop.getProperty("framework_version");
		String minSOVersion=prop.getProperty("minSOVersion");
		String minFrameworkVersion=prop.getProperty("minFrameworkVersion");
		String oldsoVersion=propold.getProperty("so_version");
		System.out.println("newFrameworkVersion is :"+framework_version+" oldFrameworkVersion is :"+oldsoVersion);
		String oldframework_version=propold.getProperty("framework_version");
		if(oldsoVersion.compareTo(minSOVersion)<0){
			//需要更新so文件
			throw new IOException("need minSOVersion is "+minSOVersion+" but now is "+oldsoVersion);
		}
		if(oldframework_version.compareTo(minFrameworkVersion)<0){
			//apkplug框架最低
			throw new IOException("need minFrameworkVersion is"+minFrameworkVersion+" but now is "+oldframework_version);
		}
		String bundle_path=prop.getProperty("bundle_path");
		String bundle_name=prop.getProperty("bundle_name");
		String classname=prop.getProperty("classname");
		File fdex=new File(appContext.getFilesDir(),bundle_name);
		if(!fdex.exists()){
			java.io.InputStream dex=jarFile.getInputStream(jarFile.getEntry(bundle_path));
			
			if(dex!=null){
				copy(dex,fdex);
			}else{
				throw new IOException("no found dex");
			}
		}
        return true;
	}
	
	public FrameworkInstance getFrame() {
		
		return frame;
	}
	 static
	    {
	    		System.loadLibrary("ndkfoo");
	    	//org/ungoverned/app/FrameworkInstance\
	    }
	 
	 public static native String FromJNI(int i);
	 public static native String appFromJNI(int i);
	 private native FrameworkInstance startFramework();
	 
	 private void copy(InputStream is, File outputFile)
		        throws IOException
		    {
		        OutputStream os = null;

		        try
		        {
		            os = new BufferedOutputStream(
		                new FileOutputStream(outputFile),4096);
		            byte[] b = new byte[4096];
		            int len = 0;
		            while ((len = is.read(b)) != -1)
		                os.write(b, 0, len);
		        }
		        finally
		        {
		            if (is != null) is.close();
		            if (os != null) os.close();
		        }
		    }
}
