package org.apkplug.app;

import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

import android.content.Context;
import android.util.DisplayMetrics;

/**
 *  框架启动接口
 * 再调用FrameworkFactory框架启动成功后将返回该接口
 * @author 梁前武 
 * www.apkplug.com
 * v2.0.0
 */
public interface FrameworkInstance {
	/**
	 * 未知状态（未启动）
	 */
	 public static final int UNKNOWN_STATUS  = -1;
	 /**
	  * 正在运行
	  */
	 public static final int RUNNING_STATUS  = 0;
	 /**
	  * 正在启动
	  */
	 public static final int STARTING_STATUS = 1;
	 /**
	  * 正在停止
	  */
	 public static final int STOPPING_STATUS = 2;
	/**
	 * 启动框架
	 * @param activatorList   启动框架时可以传人一个activator列表,作为系统级别的OSGI服务
	 * @param appContext     android的Context 建议使用ApplicationContext
	 * @param property		 框架需要保持持久化信息所需要的接口
	 * @param metrics
	 * @throws Exception
	 */
	public void start(List activatorList,Context appContext,PropertyInstance property,DisplayMetrics metrics) throws Exception;
	
	/**
	 * 停止框架
	 */
	public void shutdown();
	/**
	 * 获取框架当前状态
	 * @return
	 */
	public int getFrameworkStatus();
	/**
	 * 获取框架的SystemBundle
	 * SystemBundle是宿主与插件之间的通信桥梁
	 * 可通过SystemBundle查询框架服务,与插件通信等操作
	 * Bundle 作用参看OSGI
	 * @return
	 */
	public Bundle getSystemBundle();
	/**
	 * 获取SystemBundleContext
	 * BundleContext 作用参看OSGI
	 * @return
	 */
	public BundleContext getSystemBundleContext();
	
	public void setMetrics(DisplayMetrics metrics);
}
